package jpg2ascii;

import java.awt.*;
import java.awt.image.*;

public class Pixel{
	private String fileName, mapFileName;	
	private String mychar;
	private AsciiGenerator ascii;
	private int w,h;
	private int cps = 75; // chars per line

	// although BufferdImage is given, interpret it as Image
	public void extractPixels(Image img){
		int pixel[] = new int[w * h];
		int x = 0, y= 0;
	
		img = optimizeSize(img, cps, -1);
		//next lines are from j2sdk API
		PixelGrabber pg = new PixelGrabber(img, x, y, w, h, pixel, 0, w);
		try {
			pg.grabPixels();
		} catch (InterruptedException e) {
			System.err.println("interrupted waiting for pixels!");
			return;
		}
		if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
			System.err.println("image fetch aborted or errored");
			return;
		}

		// ascii file erstellen
		ascii = new AsciiGenerator(fileName, mapFileName);

		for (int j = 0; j < h; j++) {
			for (int i = 0; i < w; i++) {
				singlePixel(x+i, y+j, pixel[j * w + i]);
			}
			ascii.newLine();
		}	
		ascii.closeFile();
	}

	public void singlePixel(int x, int y, int pixel) {
		int alpha = (pixel >> 24) & 0xff;
		int red   = (pixel >> 16) & 0xff;
		int green = (pixel >>  8) & 0xff;
		int blue  = (pixel      ) & 0xff;
		int[] colors = {red, green, blue};

		ascii.createPixel(colors);
	}

	private Image optimizeSize(Image i, int w, int h){
		BufferedImage bi = (BufferedImage)i;
		if(bi.getWidth() != cps){
			if(w > 0 && h <= 0){
				//TODO: 2 & 3 are scale factors, ask the user
				h = (bi.getHeight()*w*1) / (2*bi.getWidth()); 
			}
			else {
				System.out.println("using standard "+cps+"*"+cps+" chars");
				w = cps;
				h = cps;
			}
			this.w = w;
			this.h = h;
			// negative height or with is for keeping aspect ratio
			i = i.getScaledInstance(w,h,i.SCALE_SMOOTH);
			System.out.println("resizing image to "+w+"*"+h+" chars");
		}
		return i;
	}

	public void	setOptions(String f, String mf, int w, int h, int cps){
		this.fileName = f;
		this.mapFileName = mf;
		this.w = w;
		this.h = h;
		this.cps = cps;
	}
}
